@@ -9,7 +9,7 @@ from dataclasses import Field, dataclass, fields
 from typing import Any, Callable, Iterator, Optional, Union
 
 from dollar_lambda.parser import Parser, defaults, flag, nonpositional, option
-from dollar_lambda.sequence import Output, Tree
+from dollar_lambda.sequence import KeyValue, Output, Sequence
 
 
 def field(
@@ -217,12 +217,11 @@ class Args:
         *args,
         flip_bools: bool = True,
         repeated: Optional[Parser[Output]] = None,
-    ) -> typing.Dict[str, Any]:
+    ) -> Optional[typing.Dict[str, Any]]:
         """
         Parses the arguments and returns a dictionary of the parsed values.
         """
-        parser = (
+        return (
             cls.parser(flip_bools=flip_bools, repeated=repeated)
-            >> Parser[Output[Tree[Any]]].done()
-        )
-        return parser.parse_args(*args)
+            >> Parser[Output[Sequence[KeyValue[Any]]]].done()
+        ).parse_args(*args)
