@@ -24,7 +24,7 @@ from dollar_lambda.error import (
     UnexpectedError,
 )
 from dollar_lambda.result import Result
-from dollar_lambda.sequence import KeyValue, Output, Sequence
+from dollar_lambda.sequence import KeyValue, Output, Sequence, TreePath
 
 A_co = TypeVar("A_co", covariant=True)
 A_monoid = TypeVar("A_monoid", bound=Monoid)
@@ -368,6 +368,10 @@ class Parser(MonadPlus[A_co]):
                 print_usage(self.usage)
             if error.usage:
                 print(error.usage)
+        if TESTING:
+            return
+        else:
+            exit()
 
     def ignore(
         self: "Parser[Output[A_monoid]]", a: Optional[Type[A_monoid]] = None
@@ -493,7 +497,7 @@ class Parser(MonadPlus[A_co]):
         *args: str,
         allow_unparsed: bool = False,
         check_help: bool = True,
-    ) -> "Dict[str, Any]":
+    ) -> "Optional[Dict[str, Any]]":
         """
         The main way the user extracts parsed results from the parser.
 
@@ -532,12 +536,10 @@ class Parser(MonadPlus[A_co]):
         result = self.parse(Sequence(list(_args))).get
         if isinstance(result, ArgumentError):
             self.handle_error(result)
-            if TESTING:
-                return  # type: ignore[return-value]
-            else:
-                exit()
-
-        return result.head.parsed.get.to_dict()
+            return None
+        get = result.head.parsed.get
+        assert isinstance(get, Sequence), get
+        return get.to_dict()
 
     @classmethod
     def return_(cls, a: A_co) -> "Parser[A_co]":  # type: ignore[misc]
@@ -591,6 +593,34 @@ class Parser(MonadPlus[A_co]):
 
         return self.apply(f)
 
+    def nesting(
+        self: "Parser[Output[Sequence[KeyValue[Any]]]]",
+    ) -> "Parser[Output[Sequence[KeyValue[Any]]]]":
+        """
+        A wrapper around `apply` that simply applies `f` to the value of the most recently parsed input.
+        >>> p1 = option("x") >> option("y")
+        >>> p = p1.type(int)
+        >>> p.parse_args("-x", "1", "-y", "2")  # converts "1" but not "2"
+        {'x': '1', 'y': 2}
+        """
+
+        def g(out: Output[Sequence[KeyValue[str]]]) -> Result[Output]:
+            d = out.get
+            if not d:
+                raise RuntimeError("Invoked nested on a parser that returns no output.")
+            *tail, head = out.get
+            if "." in head.key:
+                key, hd, *tl = head.key.split(".")
+                parents = NonemptyList.make(hd, *tl)
+                path = TreePath(parents, head.value)
+                kv = KeyValue(key, path)
+                return Result.return_(Output(Sequence([*tail, kv])))
+            else:
+                return Result.return_(out)
+
+        p = self.apply(g)
+        return replace(p, usage=self.usage, helps=self.helps)
+
     def type(
         self: "Parser[Output[Sequence[KeyValue[str]]]]", f: Callable[[str], Any]
     ) -> "Parser[Output[Sequence[KeyValue[str]]]]":
@@ -702,7 +732,10 @@ def apply(f: Callable[[str], B_monoid], description: str) -> Parser[B_monoid]:
 
 
 def argument(
-    dest: str, help: Optional[str] = None, type: Optional[Callable[[str], Any]] = None
+    dest: str,
+    nesting: bool = True,
+    help: Optional[str] = None,
+    type: Optional[Callable[[str], Any]] = None,
 ) -> Parser[Output]:
     """
     Parses a single word and binds it to `dest`.
@@ -713,23 +746,41 @@ def argument(
     dest : str
         The name of variable to bind to:
 
+    nesting : bool
+        If `True`, then the parser will split the parsed output on `.` yielding nested output.
+        See Examples for more details.
+
     help : Optional[str]
         The help message to display for the option:
 
     type : Optional[Callable[[str], Any]]
         Use the `type` argument to convert the input to a different type:
 
-    >>> argument("name").parse_args("Alice")
-    {'name': 'Alice'}
+    >>> argument("name").parse_args("Dante")
+    {'name': 'Dante'}
     >>> argument("name").parse_args()
     usage: NAME
     The following arguments are required: name
+
+    Here are some examples that take advantage of `nesting=True`:
+    >>> argument("config.name").parse_args("-h")
+    usage: CONFIG.NAME
+    >>> argument("config.name").parse_args("Dante")
+    {'config': {'name': 'Dante'}}
+
+    Of course, you can disable this by setting `nesting=False`:
+    >>> argument("config.name", nesting=False).parse_args("Dante")
+    {'config.name': 'Dante'}
+    >>> (argument("config.first.name") >> argument("config.last.name")).parse_args("Dante", "Alighieri")
+    {'config': {'first': {'name': 'Dante'}, 'last': {'name': 'Alighieri'}}}
     """
     parser = item(dest)
     _type: Callable[[str], Any] = str if type is None else type  # type: ignore[assignment]
     # Mypy doesn't know that types also have type Callable[[str], Any]
     if _type is not str:
         parser = parser.type(_type)
+    if nesting:
+        parser = parser.nesting()
     helps = {dest: help} if help else {}
     parser = replace(parser, usage=dest.upper(), helps=helps)
     return parser
@@ -811,7 +862,8 @@ def matches(
 
     def predicate(_s: str) -> bool:
         if regex:
-            return bool(re.match(s, _s))
+            b = bool(re.match(s, _s))
+            return b
         else:
             return s == _s
 
@@ -835,6 +887,7 @@ def matches(
 
 def flag(
     dest: str,
+    nesting: bool = True,
     default: Optional[bool] = None,
     help: Optional[str] = None,
     short: bool = True,
@@ -853,6 +906,10 @@ def flag(
     dest : str
         The variable to which the value will be bound.
 
+    nesting : bool
+        If `True`, then the parser will split the parsed output on `.` yielding nested output.
+        See Examples for more details.
+
     default : Optional[bool]
         An optional default value.
 
@@ -872,6 +929,9 @@ def flag(
     Here is an example using the `default` parameter:
 
     >>> p = flag("verbose", default=False)
+    >>> p.parse_args("-h")
+    usage: --verbose
+
     >>> p.parse_args()
     {'verbose': False}
 
@@ -900,6 +960,9 @@ def flag(
 
     >>> flag("value", string="v").parse_args("v")  # note that string does not have to start with -
     {'value': True}
+
+    >>> flag("config.value").parse_args("--config.value")
+    {'config': {'value': True}}
     """
     if string is None:
         _string = f"--{dest}" if len(dest) > 1 else f"-{dest}"
@@ -910,7 +973,10 @@ def flag(
         cs: Sequence[str],
         s: str,
     ) -> Result[Parse[Output[Sequence[KeyValue[bool]]]]]:
-        parser = matches(s) >= (lambda _: defaults(**{dest: not default}))
+        _defaults = defaults(**{dest: not default})
+        if nesting:
+            _defaults = _defaults.nesting()
+        parser = matches(s) >= (lambda _: _defaults)
         return parser.parse(cs)
 
     parser = Parser(partial(f, s=_string), usage=None, helps={})
@@ -939,7 +1005,7 @@ def help_parser(usage: Optional[str], parsed: A_monoid) -> Parser[A_monoid]:
 
 def item(
     name: str,
-    help_name: Optional[str] = None,
+    usage_name: Optional[str] = None,
 ) -> Parser[Output[Sequence[KeyValue[str]]]]:
     """
     Parses a single word and binds it to `dest`.
@@ -947,13 +1013,13 @@ def item(
 
     Parameters
     ----------
-    help_name : Optional[str]
-        Used for generating help text
+    usage_name : Optional[str]
+        Used for generating usage text
 
     Examples
     --------
 
-    >>> p = item("name", help_name="Your first name")
+    >>> p = item("name", usage_name="Your first name")
     >>> p.parse_args("Alice")
     {'name': 'Alice'}
     >>> p.parse_args()
@@ -977,7 +1043,7 @@ def item(
         return Result(
             MissingError(
                 missing=name,
-                usage=f"The following arguments are required: {help_name or name}",
+                usage=f"The following arguments are required: {usage_name or name}",
             )
         )
 
@@ -1066,6 +1132,7 @@ def nonpositional(
 
 def option(
     dest: str,
+    nesting: bool = True,
     flag: Optional[str] = None,
     default: Any = None,
     help: Optional[str] = None,
@@ -1080,6 +1147,10 @@ def option(
     dest : str
         The name of variable to bind to:
 
+    nesting : bool
+        If `True`, then the parser will split the parsed output on `.` yielding nested output.
+        See Examples for more details.
+
     flag : Optional[str]
         The flag to use for the option. If not provided, defaults to `--{dest}`.
 
@@ -1110,7 +1181,13 @@ def option(
 
     This example demonstrates the use of the `default` parameter:
 
-    >>> option("count", default=2).parse_args()
+    >>> p = option("count", default=2)
+    >>> p.parse_args("-h")
+    usage: --count COUNT
+    count: (default: 2)
+
+
+    >>> p.parse_args()
     {'count': 2}
 
     Here we specify a help-string using the `help` parameter:
@@ -1135,6 +1212,9 @@ def option(
     {'x': 1}
     >>> option("x", type=lambda x: int(x) + 1).parse_args("-x", "1")
     {'x': 2}
+
+    >>> option("config.x").parse_args("--config.x", "a")
+    {'config': {'x': 'a'}}
     """
 
     if flag is None:
@@ -1145,15 +1225,17 @@ def option(
     def f(
         cs: Sequence[str],
     ) -> Result[Parse[Output[Sequence[KeyValue[str]]]]]:
-        parser = matches(_flag) >= (lambda _: item(dest, help_name=dest.upper()))
+        parser = matches(_flag) >= (
+            lambda _: argument(dest, nesting=nesting, type=type)
+        )
         return parser.parse(cs)
 
     parser = Parser(f, usage=None, helps={})
-    if type is not str:
-        parser = parser.type(type)
     if short and len(dest) > 1:
         parser2 = option(dest=dest, short=False, flag=f"-{dest[0]}", default=None)
         parser = parser | parser2
+    if default:
+        help = f"{help + ' ' if help else ''}(default: {default})"
     helps = {dest: help} if help else {}
     parser = replace(parser, usage=f"{_flag} {dest.upper()}", helps=helps)
     return parser if default is None else parser | defaults(**{dest: default})
